﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace PT
{
    public partial class PTC
    {
        private string sendSerialLine, previousSendSerialLine, richTextBoxText;

        private void openSerial()
        {
            if (serialPort1.IsOpen) // close and reopen if already open
                serialPort1.Close();
            serialPort1.PortName = textBox1.Text;
            serialPort1.BaudRate = Convert.ToInt32(textBox2.Text);
            serialPort1.Open();
        }
        // sendSerial needs to be in new thread to slow down, it always reads from global sendSerialLine
        delegate void sendSerialDelegate();
        private void sendSerial()
        {
            // still printing to fast for feedback from the device fromSerial char array reads
            // "170180p now panning to -22101584" which causes the servos to jump around
            // i will attempt to print one char at a time as i designed the microcontroller code to do
            if (ready)
            {
                ready = false;

                // clear the richTextBox if to long
                richTextBoxText = richTextBox1.Text;
                if (richTextBoxText.Length > 200)
                {
                    richTextBoxText = "";
                    richTextBox1.Text = "";
                }

                if (!serialPort1.IsOpen) // if port not open open it
                    openSerial();

                // this method follows Beau's PT opertaion syntax
                char[] buffer = new char[100];
                for (int i = 0; i < 100; i++)
                    buffer[i] = '0';

                richTextBox1.SelectionColor = Color.Blue;
                richTextBox1.AppendText("\nAcutally writing.\n\n");
                richTextBox1.SelectionColor = Color.Black;
                textBox4.Text = "";

                for (int i = 0; i < sendSerialLine.Length; i++)
                {
                    //buffer[i] = sendSerialLine[i]; // for the whole line at once

                    buffer[0] = sendSerialLine[i]; // one at a time
                    Thread.Sleep(15); // 5ms more than delay on microcontroller
                    serialPort1.Write(buffer, 0, 1);
                    textBox4.AppendText(buffer[0].ToString());
                }

                //Thread.Sleep(250);
                //textBox4.AppendText(sendSerialLine);
                //serialPort1.Write(buffer, 0, sendSerialLine.Length); 

                if (sendSerialLine == "f")
                {
                    Thread.Sleep(250);
                    FPTmode = true;
                }

                ready = true;
            }
        }
        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            richTextBox1.AppendText(serialPort1.ReadExisting());
        }
        private void sendPTcommand()
        {
            dontSend = false;
            //checkSendSerialLine(); // when send serial tries to send some invalid number

            // it appears to be writing to the serial port too fast for it to handle
            // instead of writing every other time with an if and bool like before,
            // i now use sleep in one single other thread

            richTextBox1.SelectionColor = Color.Aqua;
            richTextBox1.AppendText("\nAttempting to write.\n\n");
            richTextBox1.SelectionColor = Color.Black;

            if (sendSerialLine == previousSendSerialLine)
                dontSend = true;

            previousSendSerialLine = sendSerialLine;

            if (!dontSend)
            {
                sendSerialDelegate ss = new sendSerialDelegate(sendSerial);
                ss.BeginInvoke(null, null);
            }
            else
            {
                richTextBox1.SelectionColor = Color.Maroon;
                richTextBox1.AppendText("didn't send\n\n");
                richTextBox1.SelectionColor = Color.Black;
            }
        }
        private void sendFPT(Point p)
        {
            if (FPTmode)
            {
                p.Y += 128;

                bit8[0] = Convert.ToByte(p.X);
                bit8[1] = Convert.ToByte(p.Y);
                serialPort1.Write(bit8, 0, 2);
            }
        }
        private void enterFPT()
        {
            sendSerialLine = "f";
            sendSerialDelegate ss = new sendSerialDelegate(sendSerial);
            ss.BeginInvoke(null, null);
        }
        private void exitFPT()
        {
            bit8[0] = 255;
            serialPort1.Write(bit8, 0, 1);
        }
    }
}