// beau's PT command system with FPT (fast pan tilt) mode

#include <Servo.h>

const double panRation = 1.417322;
const double tiltRation = 1.428571;

Servo pan;
Servo tilt;

int moveSpeed = 25;

int panAngle = 0;
int tiltAngle = 0;

int previousPanAngle = 0;
int previousTiltAngle = 0;

int i=0;

const int MAXCHARSFROMSERIAL = 8; // "180p108t" wont need more than 8
const int FROMSERIALSIZE = 100; // needed to avoid errors if serial flooded quickly

char fromSerial[FROMSERIALSIZE];

void setup() 
{
    pan.attach(10); // digital pin 10 - why was 14 analog pin 0?
    tilt.attach(9);
    Serial.begin(115200);
    //only enable when needed to move to avoid noise when controlling with serial monitor
    pan.detach();
    tilt.detach();

    Serial.println("startup complete");
}

void loop() 
{
    // designed to read one char at a time until one of the if chars is read and then cleared
    if(Serial.available())
    {
        delay(10);
        fromSerial[i] = Serial.read();

        //testing block
        /*
        Serial.print("\n");
        Serial.print(fromSerial);
        */

        i++;
        if(i > MAXCHARSFROMSERIAL)
            resetSerialFeed();
    }    

    if(fromSerial[i-1] == 'p') // pan
    {        
        panAngle = calculateValueUpToDelimeter();        
        if(panAngle == -1) // value invalid
            panAngle = previousPanAngle; // set it back to last good value

        if(previousPanAngle != panAngle)
        {
            //testing block
            Serial.print("\npanning now to ");
            Serial.print(panAngle);
            Serial.print(" from ");
            Serial.print(previousPanAngle);

            panSlow(previousPanAngle, panAngle, moveSpeed);
            previousPanAngle = panAngle;
        }
        else
        {
            Serial.print("\nalready in that pan position");
        }        
        resetSerialFeed();
    }
    if(fromSerial[i-1] == 't') // tilt
    {
        tiltAngle = calculateValueUpToDelimeter();
        if(tiltAngle == -1)
            tiltAngle = previousTiltAngle;

        if(previousTiltAngle != tiltAngle)
        {
            //testing block
            Serial.print("\ntilting now to ");
            Serial.print(tiltAngle);
            Serial.print(" from ");
            Serial.print(previousTiltAngle);

            tiltSlow(previousTiltAngle, tiltAngle, moveSpeed);
            previousTiltAngle = tiltAngle;
        }
        else
        {
            Serial.print("\nalready in that tilt position");
        }        
        resetSerialFeed();
    }
    if(fromSerial[i-1] == 'm') // set servo speed
    {
        moveSpeed = calculateValueUpToDelimeter();
        if(moveSpeed == -1)
            moveSpeed = 5;    
        resetSerialFeed();
    }
    if(fromSerial[i-1] == 's') // stretch
    {
        stretchServos(moveSpeed);
        resetSerialFeed();
    }
    if(fromSerial[i-1] == 'f') // fpt mode
    {
        fastPanTilt();
        resetSerialFeed();
    }
}

int calculateValueUpToDelimeter()
{
    int value = 0;
    clearArray(fromSerial, i-1, 6);
    value = atoi(fromSerial);
    if(value > 180)
    {
        //error
        return -1;
    }
    else if(value < 0)
    {
        //error
        return -1;
    }
    else
    {
        return value;
    }
}

void resetSerialFeed()
{
    /*
    while(Serial.available()) // read all the chars the delete them to clear the buffer
    {
        fromSerial[i] = Serial.read();
    }
    */
    i=0;
    clearArray(fromSerial, 0, FROMSERIALSIZE);
    Serial.flush();
}

void fillArray(char ar[], int size)
{
    for(int i=0; i<size; i++)
    {
        ar[i] = '0';
    }
}

void clearArray(char ar[], int begin, int end)
{
    for(int i=begin; i<end; i++)
    {
        ar[i] = NULL;
    }
}

void panSlow(int previousAngle, int currentAngle, int milliseconds)
{
    
    pan.attach(10);
    if(previousAngle <= currentAngle)
    {
        for(int angle = previousAngle; angle <= currentAngle; angle++)
        {
            pan.write(angle);
            delay(milliseconds);
        }
    }
    else
    {
        for(int angle = previousAngle; angle >= currentAngle; angle--)
        {
            pan.write(angle);
            delay(milliseconds);
        }
    }  

    
    for(int i=0; i<5; i++) // write a few more times to ensure right position 50ms
    {
        delay(5);
        pan.write(currentAngle);
        delay(5);
    }    

    pan.detach();
}

void tiltSlow(int previousAngle, int currentAngle, int milliseconds)
{
    
    tilt.attach(9);
    if(previousAngle <= currentAngle)
    {
        for(int angle = previousAngle; angle <= currentAngle; angle++) // <= should write last value twice just in case
        {
            tilt.write(angle);
            delay(milliseconds);
        }
    }
    else
    {
        for(int angle = previousAngle; angle >= currentAngle; angle--) // >= should write last value twice just in case
        {
            tilt.write(angle);
            delay(milliseconds);
        }
    }    

    
    for(int i=0; i<5; i++) // write a few more times to ensure right position 50ms
    {
        delay(5);
        tilt.write(currentAngle);
        delay(5);
    }    

    tilt.detach();
}

void stretchServos(int speed)
{
    if(panAngle > 0 || tiltAngle > 0)
    {
        panSlow(panAngle, 0, speed);
        tiltSlow(tiltAngle, 0, speed);        
    }
    pan.attach(10);
    tilt.attach(9);
    for(int angle = 0; angle < 180; angle++)
    {
        pan.write(angle);
        delay(speed);
        tilt.write(angle);
        delay(speed);
    }
    for(int angle = 180; angle > 0; angle--)
    {
        pan.write(angle);
        delay(speed);
        tilt.write(angle);
        delay(speed);
    }
    pan.detach();
    tilt.detach();
    panAngle = previousPanAngle = tiltAngle = previousTiltAngle = 0;
}

void fastPanTilt()
{
    bool FPT = true;
    int angle = 0;

    pan.attach(10);
    tilt.attach(9);

    while(FPT)
    {
        while(Serial.available())
        {
            int byte = Serial.read();
            if(byte == 255) // leave FPT mode
            {
                FPT = false;
                break;
            }
            else if(byte <= 127)
            {
                angle = byte * panRation;
                pan.write(angle);
                /*
                Serial.print("pan: ");
                Serial.print(angle);
                Serial.print("\n");
                */
            }
            else // (byte > 127 && byte < 255)
            {
                angle = (byte-128) * tiltRation;
                tilt.write(angle);
                /*
                Serial.print("tilt: ");
                Serial.print(angle);
                Serial.print("\n");
                */
            }            
        }
    }

    pan.detach();
    tilt.detach();
}